@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@ require_once dirname(__FILE__).'/include/discovery.inc.php';
 $page['title'] = _('Configuration of discovery rules');
 $page['file'] = 'discoveryconf.php';
 $page['type'] = detect_page_type();
+$page['scripts'] = ['class.cviewswitcher.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -40,6 +41,8 @@ $fields = [
 	],
 	'status' =>			[T_ZBX_INT, O_OPT, null,	IN('0,1'),	null],
 	'uniqueness_criteria' => [T_ZBX_STR, O_OPT, null, null,	'isset({add}) || isset({update})', _('Device uniqueness criteria')],
+	'host_source' =>	[T_ZBX_STR, O_OPT, null,	null,	null],
+	'name_source' =>	[T_ZBX_STR, O_OPT, null,	null,	null],
 	'g_druleid' =>		[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
 	'dchecks' =>		[null, O_OPT, null,		null,		null],
 	// actions
@@ -80,7 +83,7 @@ if (isset($_REQUEST['druleid'])) {
 		'selectDChecks' => [
 			'type', 'key_', 'snmp_community', 'ports', 'snmpv3_securityname', 'snmpv3_securitylevel',
 			'snmpv3_authpassphrase', 'snmpv3_privpassphrase', 'uniq', 'snmpv3_authprotocol', 'snmpv3_privprotocol',
-			'snmpv3_contextname'
+			'snmpv3_contextname', 'host_source', 'name_source'
 		],
 		'editable' => true
 	]);
@@ -89,38 +92,6 @@ if (isset($_REQUEST['druleid'])) {
 	}
 }
 
-// ajax
-if (isset($_REQUEST['output']) && $_REQUEST['output'] == 'ajax') {
-	$ajaxResponse = new CAjaxResponse;
-
-	if (isset($_REQUEST['ajaxaction']) && $_REQUEST['ajaxaction'] == 'validate') {
-		$ajaxData = getRequest('ajaxdata', []);
-		$item_key_parser = new CItemKey();
-
-		foreach ($ajaxData as $check) {
-			switch ($check['field']) {
-				case 'port':
-					if (!validate_port_list($check['value'])) {
-						$ajaxResponse->error(_('Incorrect port range.'));
-					}
-					break;
-				case 'itemKey':
-					if ($item_key_parser->parse($check['value']) != CParser::PARSE_SUCCESS) {
-						$ajaxResponse->error(
-							_s('Invalid key "%1$s": %2$s.', $check['value'], $item_key_parser->getError())
-						);
-					}
-					break;
-			}
-		}
-	}
-
-	$ajaxResponse->send();
-
-	require_once dirname(__FILE__).'/include/page_footer.php';
-	exit;
-}
-
 /*
  * Action
  */
@@ -228,6 +199,15 @@ elseif (hasRequest('action') && getRequest('action') == 'drule.massdelete' && ha
 	show_messages($result, _('Discovery rules deleted'), _('Cannot delete discovery rules'));
 }
 
+if (hasRequest('action') && hasRequest('g_druleid') && !$result) {
+	$drules = API::DRule()->get([
+		'output' => [],
+		'druleids' => getRequest('g_druleid'),
+		'editable' => true
+	]);
+	uncheckTableRows(null, zbx_objectValues($drules, 'druleid'));
+}
+
 /*
  * Display
  */
@@ -243,12 +223,26 @@ if (isset($_REQUEST['form'])) {
 	if (isset($data['druleid']) && !isset($_REQUEST['form_refresh'])) {
 		$data['drule'] = reset($dbDRule);
 		$data['drule']['uniqueness_criteria'] = -1;
+		$data['drule']['host_source'] = ZBX_DISCOVERY_DNS;
+		$data['drule']['name_source'] = ZBX_DISCOVERY_UNSPEC;
 
 		if (!empty($data['drule']['dchecks'])) {
+			$dcheck = reset($data['drule']['dchecks']);
+			$data['drule']['host_source'] = $dcheck['host_source'];
+			$data['drule']['name_source'] = $dcheck['name_source'];
+
 			foreach ($data['drule']['dchecks'] as $id => $dcheck) {
 				if ($dcheck['uniq']) {
 					$data['drule']['uniqueness_criteria'] = $dcheck['dcheckid'];
 				}
+
+				if ($dcheck['host_source'] == ZBX_DISCOVERY_VALUE) {
+					$data['drule']['host_source'] = '_'.$dcheck['dcheckid'];
+				}
+
+				if ($dcheck['name_source'] == ZBX_DISCOVERY_VALUE) {
+					$data['drule']['name_source'] = '_'.$dcheck['dcheckid'];
+				}
 			}
 		}
 	}
@@ -261,16 +255,62 @@ if (isset($_REQUEST['form'])) {
 		$data['drule']['dchecks'] = getRequest('dchecks', []);
 		$data['drule']['nextcheck'] = getRequest('nextcheck', 0);
 		$data['drule']['uniqueness_criteria'] = getRequest('uniqueness_criteria', -1);
+		$data['drule']['host_source'] = getRequest('host_source', ZBX_DISCOVERY_DNS);
+		$data['drule']['name_source'] = getRequest('name_source', ZBX_DISCOVERY_UNSPEC);
 	}
 
 	if (!empty($data['drule']['dchecks'])) {
-		foreach ($data['drule']['dchecks'] as $id => $dcheck) {
-			$data['drule']['dchecks'][$id]['name'] = discovery_check2str(
-				$dcheck['type'],
-				isset($dcheck['key_']) ? $dcheck['key_'] : '',
-				isset($dcheck['ports']) ? $dcheck['ports'] : ''
+		$data['drule']['dchecks'] = array_map(function($value) {
+			$data = [
+				'type' => $value['type'],
+				'dcheckid' => $value['dcheckid'],
+				'ports' => $value['ports'],
+				'uniq' => array_key_exists('uniq', $value) ? $value['uniq'] : null,
+				'host_source' => $value['host_source'],
+				'name_source' => $value['name_source']
+			];
+
+			$data['name'] = discovery_check2str(
+				$value['type'],
+				isset($value['key_']) ? $value['key_'] : '',
+				isset($value['ports']) ? $value['ports'] : ''
 			);
-		}
+
+			switch($value['type']) {
+				case SVC_SNMPv1:
+				case SVC_SNMPv2c:
+					$data['snmp_community'] = $value['snmp_community'];
+					// break; is not missing here
+				case SVC_AGENT:
+					$data['key_'] = $value['key_'];
+					break;
+				case SVC_SNMPv3:
+					$data += [
+						'key_' => $value['key_'],
+						'snmpv3_contextname' => $value['snmpv3_contextname'],
+						'snmpv3_securityname' => $value['snmpv3_securityname'],
+						'snmpv3_securitylevel' => $value['snmpv3_securitylevel'],
+					];
+
+					if ($value['snmpv3_securitylevel'] == ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV
+							|| $value['snmpv3_securitylevel'] == ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV) {
+						$data += [
+							'snmpv3_authprotocol' => $value['snmpv3_authprotocol'],
+							'snmpv3_authpassphrase' => $value['snmpv3_authpassphrase']
+						];
+					}
+
+					if ($value['snmpv3_securitylevel'] == ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV) {
+						$data += [
+							'snmpv3_privprotocol' => $value['snmpv3_privprotocol'],
+							'snmpv3_privpassphrase' => $value['snmpv3_privpassphrase']
+						];
+					}
+					break;
+			}
+
+			return $data;
+		}, $data['drule']['dchecks']);
 
 		order_result($data['drule']['dchecks'], 'name');
 	}
@@ -282,9 +322,7 @@ if (isset($_REQUEST['form'])) {
 	order_result($data['proxies'], 'host');
 
 	// render view
-	$discoveryView = new CView('configuration.discovery.edit', $data);
-	$discoveryView->render();
-	$discoveryView->show();
+	echo (new CView('configuration.discovery.edit', $data))->getOutput();
 }
 else {
 	$sortField = getRequest('sort', CProfile::get('web.'.$page['file'].'.sort', 'name'));
@@ -335,7 +373,6 @@ else {
 
 	if ($data['drules']) {
 		foreach ($data['drules'] as $key => $drule) {
-			// checks
 			$checks = [];
 
 			foreach ($drule['dchecks'] as $check) {
@@ -346,26 +383,31 @@ else {
 
 			$data['drules'][$key]['checks'] = $checks;
 
-			// description
-			$data['drules'][$key]['description'] = [];
-
-			if ($drule['proxy_hostid']) {
-				$proxy = get_host_by_hostid($drule['proxy_hostid']);
-
-				array_push($data['drules'][$key]['description'], $proxy['host'].NAME_DELIMITER);
-			}
+			$data['drules'][$key]['proxy'] = ($drule['proxy_hostid'] != 0)
+				? get_host_by_hostid($drule['proxy_hostid'])['host']
+				: '';
 		}
 
 		order_result($data['drules'], $sortField, $sortOrder);
 	}
 
-	// get paging
-	$data['paging'] = getPagingLine($data['drules'], $sortOrder, new CUrl('discoveryconf.php'));
+	// pager
+	if (hasRequest('page')) {
+		$page_num = getRequest('page');
+	}
+	elseif (isRequestMethod('get') && !hasRequest('cancel')) {
+		$page_num = 1;
+	}
+	else {
+		$page_num = CPagerHelper::loadPage($page['file']);
+	}
+
+	CPagerHelper::savePage($page['file'], $page_num);
+
+	$data['paging'] = CPagerHelper::paginate($page_num, $data['drules'], $sortOrder, new CUrl('discoveryconf.php'));
 
 	// render view
-	$discoveryView = new CView('configuration.discovery.list', $data);
-	$discoveryView->render();
-	$discoveryView->show();
+	echo (new CView('configuration.discovery.list', $data))->getOutput();
 }
 
 require_once dirname(__FILE__).'/include/page_footer.php';
