@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -48,7 +48,6 @@ abstract class CItemGeneral extends CApiService {
 		// host - value should be null for template items
 		$this->fieldRules = [
 			'type'					=> ['template' => 1],
-			'snmp_community'		=> [],
 			'snmp_oid'				=> ['template' => 1],
 			'hostid'				=> [],
 			'name'					=> ['template' => 1],
@@ -59,16 +58,10 @@ abstract class CItemGeneral extends CApiService {
 			'history'				=> [],
 			'trends'				=> [],
 			'status'				=> [],
+			'discover'				=> [],
 			'value_type'			=> ['template' => 1],
 			'trapper_hosts'			=> [],
 			'units'					=> ['template' => 1],
-			'snmpv3_contextname'	=> [],
-			'snmpv3_securityname'	=> [],
-			'snmpv3_securitylevel'	=> [],
-			'snmpv3_authprotocol'	=> [],
-			'snmpv3_authpassphrase'	=> [],
-			'snmpv3_privprotocol'	=> [],
-			'snmpv3_privpassphrase'	=> [],
 			'formula'				=> ['template' => 1],
 			'error'					=> ['system' => 1],
 			'lastlogsize'			=> ['system' => 1],
@@ -86,10 +79,10 @@ abstract class CItemGeneral extends CApiService {
 			'flags'					=> [],
 			'filter'				=> [],
 			'interfaceid'			=> ['host' => 1],
-			'port'					=> [],
 			'inventory_link'		=> [],
 			'lifetime'				=> [],
 			'preprocessing'			=> ['template' => 1],
+			'overrides'				=> ['template' => 1],
 			'jmx_endpoint'			=> [],
 			'url'					=> ['template' => 1],
 			'timeout'				=> ['template' => 1],
@@ -331,49 +324,7 @@ abstract class CItemGeneral extends CApiService {
 
 			// Validate update interval.
 			if (!in_array($fullItem['type'], [ITEM_TYPE_TRAPPER, ITEM_TYPE_SNMPTRAP, ITEM_TYPE_DEPENDENT])) {
-				if ($update_interval_parser->parse($fullItem['delay']) != CParser::PARSE_SUCCESS) {
-					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('Incorrect value for field "%1$s": %2$s.', 'delay', _('invalid delay'))
-					);
-				}
-
-				$delay = $update_interval_parser->getDelay();
-
-				// Check if not macros. If delay is a macro, skip this step, otherwise check if delay is valid.
-				if ($delay[0] !== '{') {
-					$delay_sec = timeUnitToSeconds($delay);
-					$intervals = $update_interval_parser->getIntervals();
-					$flexible_intervals = $update_interval_parser->getIntervals(ITEM_DELAY_FLEXIBLE);
-					$has_scheduling_intervals = (bool) $update_interval_parser->getIntervals(ITEM_DELAY_SCHEDULING);
-					$has_macros = false;
-
-					foreach ($intervals as $interval) {
-						if (strpos($interval['interval'], '{') !== false) {
-							$has_macros = true;
-							break;
-						}
-					}
-
-					// If delay is 0, there must be at least one either flexible or scheduling interval.
-					if ($delay_sec == 0 && !$intervals) {
-						self::exception(ZBX_API_ERROR_PARAMETERS,
-							_('Item will not be refreshed. Specified update interval requires having at least one either flexible or scheduling interval.')
-						);
-					}
-					elseif ($delay_sec < 0 || $delay_sec > SEC_PER_DAY) {
-						self::exception(ZBX_API_ERROR_PARAMETERS,
-							_('Item will not be refreshed. Update interval should be between 1s and 1d. Also Scheduled/Flexible intervals can be used.')
-						);
-					}
-
-					// If there are scheduling intervals or intervals with macros, skip the next check calculation.
-					if (!$has_macros && !$has_scheduling_intervals && $flexible_intervals
-							&& calculateItemNextCheck(0, $delay_sec, $flexible_intervals, time()) == ZBX_JAN_2038) {
-						self::exception(ZBX_API_ERROR_PARAMETERS,
-							_('Item will not be refreshed. Please enter a correct update interval.')
-						);
-					}
-				}
+				$this->validateDelay($update_interval_parser, $fullItem['delay']);
 			}
 
 			// For non-numeric types, whichever value was entered in trends field, is overwritten to zero.
@@ -533,20 +484,28 @@ abstract class CItemGeneral extends CApiService {
 
 			// ssh, telnet
 			if ($fullItem['type'] == ITEM_TYPE_SSH || $fullItem['type'] == ITEM_TYPE_TELNET) {
-				if (zbx_empty($fullItem['username'])) {
+				if ($fullItem['username'] === '') {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _('No authentication user name specified.'));
 				}
 
 				if ($fullItem['type'] == ITEM_TYPE_SSH && $fullItem['authtype'] == ITEM_AUTHTYPE_PUBLICKEY) {
-					if (zbx_empty($fullItem['publickey'])) {
+					if ($fullItem['publickey'] === '') {
 						self::exception(ZBX_API_ERROR_PARAMETERS, _('No public key file specified.'));
 					}
-					if (zbx_empty($fullItem['privatekey'])) {
+					if ($fullItem['privatekey'] === '') {
 						self::exception(ZBX_API_ERROR_PARAMETERS, _('No private key file specified.'));
 					}
 				}
 			}
 
+			// Prevent IPMI sensor field being empty if item key is not "ipmi.get".
+			if ($fullItem['type'] == ITEM_TYPE_IPMI && $fullItem['key_'] !== 'ipmi.get'
+					&& (!array_key_exists('ipmi_sensor', $fullItem) || $fullItem['ipmi_sensor'] === '')) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
+					'ipmi_sensor', _('cannot be empty')
+				));
+			}
+
 			// snmp trap
 			if ($fullItem['type'] == ITEM_TYPE_SNMPTRAP
 					&& $fullItem['key_'] !== 'snmptrap.fallback' && $item_key_parser->getKey() !== 'snmptrap') {
@@ -554,43 +513,11 @@ abstract class CItemGeneral extends CApiService {
 			}
 
 			// snmp oid
-			if ((in_array($fullItem['type'], [ITEM_TYPE_SNMPV1, ITEM_TYPE_SNMPV2C, ITEM_TYPE_SNMPV3]))
-					&& zbx_empty($fullItem['snmp_oid'])) {
+			if ($fullItem['type'] == ITEM_TYPE_SNMP
+					&& (!array_key_exists('snmp_oid', $fullItem) || $fullItem['snmp_oid'] === '')) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('No SNMP OID specified.'));
 			}
 
-			// snmp community
-			if (in_array($fullItem['type'], [ITEM_TYPE_SNMPV1, ITEM_TYPE_SNMPV2C])
-					&& zbx_empty($fullItem['snmp_community'])) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('No SNMP community specified.'));
-			}
-
-			// snmp port
-			if (isset($fullItem['port']) && !zbx_empty($fullItem['port']) && !validatePortNumberOrMacro($fullItem['port'])) {
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('Item "%1$s:%2$s" has invalid port: "%3$s".', $fullItem['name'], $fullItem['key_'], $fullItem['port']));
-			}
-
-			if (isset($fullItem['snmpv3_securitylevel']) && $fullItem['snmpv3_securitylevel'] != ITEM_SNMPV3_SECURITYLEVEL_NOAUTHNOPRIV) {
-				// snmpv3 authprotocol
-				if (str_in_array($fullItem['snmpv3_securitylevel'], [ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV, ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV])) {
-					if (isset($fullItem['snmpv3_authprotocol']) && (zbx_empty($fullItem['snmpv3_authprotocol'])
-							|| !str_in_array($fullItem['snmpv3_authprotocol'],
-								[ITEM_AUTHPROTOCOL_MD5, ITEM_AUTHPROTOCOL_SHA]))) {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect authentication protocol for item "%1$s".', $fullItem['name']));
-					}
-				}
-
-				// snmpv3 privprotocol
-				if ($fullItem['snmpv3_securitylevel'] == ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV) {
-					if (isset($fullItem['snmpv3_privprotocol']) && (zbx_empty($fullItem['snmpv3_privprotocol'])
-							|| !str_in_array($fullItem['snmpv3_privprotocol'],
-								[ITEM_PRIVPROTOCOL_DES, ITEM_PRIVPROTOCOL_AES]))) {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect privacy protocol for item "%1$s".', $fullItem['name']));
-					}
-				}
-			}
-
 			if (isset($item['applications']) && $item['applications']) {
 				/*
 				 * 'flags' is available for update and item prototypes.
@@ -1235,7 +1162,8 @@ abstract class CItemGeneral extends CApiService {
 	 *                                                                  21 - ZBX_PREPROC_SCRIPT;
 	 *                                                                  22 - ZBX_PREPROC_PROMETHEUS_PATTERN;
 	 *                                                                  23 - ZBX_PREPROC_PROMETHEUS_TO_JSON;
-	 *                                                                  24 - ZBX_PREPROC_CSV_TO_JSON.
+	 *                                                                  24 - ZBX_PREPROC_CSV_TO_JSON;
+	 *                                                                  25 - ZBX_PREPROC_STR_REPLACE.
 	 * @param string $item['preprocessing'][]['params']                Additional parameters used by preprocessing
 	 *                                                                 option. Multiple parameters are separated by LF
 	 *                                                                 (\n) character.
@@ -1351,6 +1279,7 @@ abstract class CItemGeneral extends CApiService {
 
 					case ZBX_PREPROC_REGSUB:
 					case ZBX_PREPROC_ERROR_FIELD_REGEX:
+					case ZBX_PREPROC_STR_REPLACE:
 						// Check if 'params' are not empty and if second parameter contains (after \n) is not empty.
 						if (is_array($preprocessing['params'])) {
 							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
@@ -1370,7 +1299,9 @@ abstract class CItemGeneral extends CApiService {
 							));
 						}
 
-						if (!array_key_exists(1, $params) || $params[1] === '') {
+						if (($preprocessing['type'] == ZBX_PREPROC_REGSUB
+								|| $preprocessing['type'] == ZBX_PREPROC_ERROR_FIELD_REGEX)
+								&& (!array_key_exists(1, $params) || $params[1] === '')) {
 							self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
 								'params', _('second parameter is expected')
 							));
@@ -1605,6 +1536,7 @@ abstract class CItemGeneral extends CApiService {
 					case ZBX_PREPROC_THROTTLE_VALUE:
 					case ZBX_PREPROC_THROTTLE_TIMED_VALUE:
 					case ZBX_PREPROC_SCRIPT:
+					case ZBX_PREPROC_STR_REPLACE:
 						if (is_array($preprocessing['error_handler'])) {
 							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
 						}
@@ -2041,7 +1973,7 @@ abstract class CItemGeneral extends CApiService {
 	}
 
 	/**
-	 * Validate depth and ammount of elements in the tree of the dependent items.
+	 * Validate depth and amount of elements in the tree of the dependent items.
 	 *
 	 * @param array  $dependent_items
 	 * @param string $dependent_items[<master_itemid>][]  List if the dependent item IDs ("false" for new items)
@@ -2227,7 +2159,7 @@ abstract class CItemGeneral extends CApiService {
 
 		if (array_key_exists('templateid', $data) && $data['templateid']) {
 			$rules['interfaceid'] = [
-				'type' => API_INT32, 'flags' => API_REQUIRED | API_NOT_EMPTY
+				'type' => API_ID, 'flags' => API_REQUIRED | API_NOT_EMPTY
 			];
 		}
 
@@ -2245,8 +2177,6 @@ abstract class CItemGeneral extends CApiService {
 			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
 
-		$json = new CJson();
-
 		if (array_key_exists('query_fields', $item)) {
 			if (!is_array($item['query_fields'])) {
 				self::exception(ZBX_API_ERROR_PARAMETERS,
@@ -2262,9 +2192,7 @@ abstract class CItemGeneral extends CApiService {
 				}
 			}
 
-			$json_string = $json->encode($item['query_fields']);
-
-			if (strlen($json_string) > DB::getFieldLength('items', 'query_fields')) {
+			if (strlen(json_encode($item['query_fields'])) > DB::getFieldLength('items', 'query_fields')) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.', 'query_fields',
 					_('cannot convert to JSON, result value too long')
 				));
@@ -2352,9 +2280,9 @@ abstract class CItemGeneral extends CApiService {
 					$shift = $shift + 1 - strlen($substr);
 				}
 
-				$json->decode($posts);
+				json_decode($posts);
 
-				if ($json->hasError()) {
+				if (json_last_error()) {
 					self::exception(ZBX_API_ERROR_PARAMETERS,
 						_s('Invalid parameter "%1$s": %2$s.', 'posts', _('JSON is expected'))
 					);
@@ -2362,4 +2290,57 @@ abstract class CItemGeneral extends CApiService {
 			}
 		}
 	}
+
+	/**
+	 * Validates update interval.
+	 *
+	 * @param CUpdateIntervalParser $parser  Parser used for delay validation.
+	 * @param string                $value   Update interval to parse and validate.
+	 *
+	 * @throws APIException if update interval is incorrect.
+	 */
+	protected function validateDelay(CUpdateIntervalParser $parser, $value) {
+		if ($parser->parse($value) != CParser::PARSE_SUCCESS) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Incorrect value for field "%1$s": %2$s.', 'delay', _('invalid delay'))
+			);
+		}
+
+		$delay = $parser->getDelay();
+
+		if ($delay[0] !== '{') {
+			$delay_sec = timeUnitToSeconds($delay);
+			$intervals = $parser->getIntervals();
+			$flexible_intervals = $parser->getIntervals(ITEM_DELAY_FLEXIBLE);
+			$has_scheduling_intervals = (bool) $parser->getIntervals(ITEM_DELAY_SCHEDULING);
+			$has_macros = false;
+
+			foreach ($intervals as $interval) {
+				if (strpos($interval['interval'], '{') !== false) {
+					$has_macros = true;
+					break;
+				}
+			}
+
+			// If delay is 0, there must be at least one either flexible or scheduling interval.
+			if ($delay_sec == 0 && !$intervals) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_('Item will not be refreshed. Specified update interval requires having at least one either flexible or scheduling interval.')
+				);
+			}
+			elseif ($delay_sec < 0 || $delay_sec > SEC_PER_DAY) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_('Item will not be refreshed. Update interval should be between 1s and 1d. Also Scheduled/Flexible intervals can be used.')
+				);
+			}
+
+			// If there are scheduling intervals or intervals with macros, skip the next check calculation.
+			if (!$has_macros && !$has_scheduling_intervals && $flexible_intervals
+					&& calculateItemNextCheck(0, $delay_sec, $flexible_intervals, time()) == ZBX_JAN_2038) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_('Item will not be refreshed. Please enter a correct update interval.')
+				);
+			}
+		}
+	}
 }
