@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -38,77 +38,93 @@ class CScript extends CApiService {
 	protected $parent_host_groups = [];
 
 	/**
-	 * Get scripts data.
+	 * @param array $options
 	 *
-	 * @param array  $options
-	 * @param array  $options['itemids']
-	 * @param array  $options['hostids']	deprecated (very slow)
-	 * @param array  $options['groupids']
-	 * @param array  $options['triggerids']
-	 * @param array  $options['scriptids']
-	 * @param bool   $options['status']
-	 * @param bool   $options['editable']
-	 * @param bool   $options['count']
-	 * @param string $options['pattern']
-	 * @param int    $options['limit']
-	 * @param string $options['order']
+	 * @throws APIException if the input is invalid.
 	 *
-	 * @return array
+	 * @return array|int
 	 */
 	public function get(array $options) {
-		$result = [];
-
-		$sqlParts = [
-			'select'	=> ['scripts' => 's.scriptid'],
-			'from'		=> ['scripts s'],
-			'where'		=> [],
-			'order'		=> [],
-			'limit'		=> null
+		$script_fields = ['scriptid', 'name', 'command', 'host_access', 'usrgrpid', 'groupid', 'description',
+			'confirmation', 'type', 'execute_on'
+		];
+		$group_fields = ['groupid', 'name', 'flags', 'internal'];
+		$host_fields = ['hostid', 'host', 'name', 'description', 'status', 'proxy_hostid', 'inventory_mode', 'flags',
+			'available', 'snmp_available', 'jmx_available', 'ipmi_available', 'error', 'snmp_error', 'jmx_error',
+			'ipmi_error', 'errors_from', 'snmp_errors_from', 'jmx_errors_from', 'ipmi_errors_from', 'disable_until',
+			'snmp_disable_until', 'jmx_disable_until', 'ipmi_disable_until', 'ipmi_authtype', 'ipmi_privilege',
+			'ipmi_username', 'ipmi_password', 'maintenanceid', 'maintenance_status', 'maintenance_type',
+			'maintenance_from', 'tls_connect', 'tls_accept', 'tls_issuer', 'tls_subject', 'tls_psk_identity', 'tls_psk'
 		];
 
-		$defOptions = [
-			'groupids'				=> null,
-			'hostids'				=> null,
-			'scriptids'				=> null,
-			'usrgrpids'				=> null,
-			'editable'				=> false,
-			'nopermissions'			=> null,
+		$api_input_rules = ['type' => API_OBJECT, 'fields' => [
 			// filter
-			'filter'				=> null,
-			'search'				=> null,
-			'searchByAny'			=> null,
-			'startSearch'			=> false,
-			'excludeSearch'			=> false,
-			'searchWildcardsEnabled'=> null,
+			'scriptids' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'default' => null],
+			'hostids' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'default' => null],
+			'groupids' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'default' => null],
+			'usrgrpids' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'default' => null],
+			'filter' =>					['type' => API_OBJECT, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => [
+				'scriptid' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
+				'name' =>					['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
+				'command' =>				['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
+				'host_access' =>			['type' => API_INTS32, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'in' => implode(',', [PERM_READ, PERM_READ_WRITE])],
+				'usrgrpid' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
+				'groupid' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
+				'confirmation' =>			['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
+				'type' =>					['type' => API_INTS32, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'in' => implode(',', [ZBX_SCRIPT_TYPE_CUSTOM_SCRIPT, ZBX_SCRIPT_TYPE_IPMI])],
+				'execute_on' =>				['type' => API_INTS32, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'in' => implode(',', [ZBX_SCRIPT_EXECUTE_ON_AGENT, ZBX_SCRIPT_EXECUTE_ON_SERVER, ZBX_SCRIPT_EXECUTE_ON_PROXY])]
+			]],
+			'search' =>					['type' => API_OBJECT, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => [
+				'name' =>					['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
+				'command' =>				['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
+				'description' =>			['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
+				'confirmation' =>			['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE]
+			]],
+			'searchByAny' =>			['type' => API_BOOLEAN, 'default' => false],
+			'startSearch' =>			['type' => API_FLAG, 'default' => false],
+			'excludeSearch' =>			['type' => API_FLAG, 'default' => false],
+			'searchWildcardsEnabled' =>	['type' => API_BOOLEAN, 'default' => false],
 			// output
-			'output'				=> API_OUTPUT_EXTEND,
-			'selectGroups'			=> null,
-			'selectHosts'			=> null,
-			'countOutput'			=> false,
-			'preservekeys'			=> false,
-			'sortfield'				=> '',
-			'sortorder'				=> '',
-			'limit'					=> null
+			'output' =>					['type' => API_OUTPUT, 'in' => implode(',', $script_fields), 'default' => API_OUTPUT_EXTEND],
+			'selectGroups' =>			['type' => API_OUTPUT, 'flags' => API_ALLOW_NULL, 'in' => implode(',', $group_fields), 'default' => null],
+			'selectHosts' =>			['type' => API_OUTPUT, 'flags' => API_ALLOW_NULL, 'in' => implode(',', $host_fields), 'default' => null],
+			'countOutput' =>			['type' => API_FLAG, 'default' => false],
+			// sort and limit
+			'sortfield' =>				['type' => API_STRINGS_UTF8, 'flags' => API_NORMALIZE, 'in' => implode(',', $this->sortColumns), 'uniq' => true, 'default' => []],
+			'sortorder' =>				['type' => API_SORTORDER, 'default' => []],
+			'limit' =>					['type' => API_INT32, 'flags' => API_ALLOW_NULL, 'in' => '1:'.ZBX_MAX_INT32, 'default' => null],
+			// flags
+			'editable' =>				['type' => API_BOOLEAN, 'default' => false],
+			'preservekeys' =>			['type' => API_BOOLEAN, 'default' => false]
+		]];
+		if (!CApiInputValidator::validate($api_input_rules, $options, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+
+		$sql_parts = [
+			'select'	=> ['scripts' => 's.scriptid'],
+			'from'		=> ['scripts' => 'scripts s'],
+			'where'		=> [],
+			'order'		=> []
 		];
-		$options = zbx_array_merge($defOptions, $options);
 
 		// editable + permission check
-		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
 			if ($options['editable']) {
-				return $result;
+				return $options['countOutput'] ? 0 : [];
 			}
 
-			$userGroups = getUserGroupsByUserId(self::$userData['userid']);
+			$user_groups = getUserGroupsByUserId(self::$userData['userid']);
 
-			$sqlParts['where'][] = '(s.usrgrpid IS NULL OR '.dbConditionInt('s.usrgrpid', $userGroups).')';
-			$sqlParts['where'][] = '(s.groupid IS NULL OR EXISTS ('.
-					'SELECT NULL'.
-					' FROM rights r'.
-					' WHERE s.groupid=r.id'.
-						' AND '.dbConditionInt('r.groupid', $userGroups).
-					' GROUP BY r.id'.
-					' HAVING MIN(r.permission)>'.PERM_DENY.
-					'))';
+			$sql_parts['where'][] = '(s.usrgrpid IS NULL OR '.dbConditionInt('s.usrgrpid', $user_groups).')';
+			$sql_parts['where'][] = '(s.groupid IS NULL OR EXISTS ('.
+				'SELECT NULL'.
+				' FROM rights r'.
+				' WHERE s.groupid=r.id'.
+					' AND '.dbConditionInt('r.groupid', $user_groups).
+				' GROUP BY r.id'.
+				' HAVING MIN(r.permission)>'.PERM_DENY.
+			'))';
 		}
 
 		$host_groups = null;
@@ -117,7 +133,6 @@ class CScript extends CApiService {
 
 		// Hostids and groupids selection API calls must be made separately because we must intersect enriched groupids.
 		if ($options['hostids'] !== null) {
-			zbx_value2array($options['hostids']);
 			$host_groups_by_hostids = enrichParentGroups(API::HostGroup()->get([
 				'output' => ['groupid', 'name'],
 				'hostids' => $options['hostids'],
@@ -125,7 +140,6 @@ class CScript extends CApiService {
 			]));
 		}
 		if ($options['groupids'] !== null) {
-			zbx_value2array($options['groupids']);
 			$host_groups_by_groupids = enrichParentGroups(API::HostGroup()->get([
 				'output' => ['groupid', 'name'],
 				'groupids' => $options['groupids'],
@@ -144,66 +158,57 @@ class CScript extends CApiService {
 		}
 
 		if ($host_groups !== null) {
-			$sqlParts['where'][] = '('.dbConditionInt('s.groupid', array_keys($host_groups)).' OR s.groupid IS NULL)';
+			$sql_parts['where'][] = '('.dbConditionInt('s.groupid', array_keys($host_groups)).' OR s.groupid IS NULL)';
 			$this->parent_host_groups = $host_groups;
 		}
 
 		// usrgrpids
 		if ($options['usrgrpids'] !== null) {
-			zbx_value2array($options['usrgrpids']);
-
-			$sqlParts['where'][] = '(s.usrgrpid IS NULL OR '.dbConditionInt('s.usrgrpid', $options['usrgrpids']).')';
+			$sql_parts['where'][] = '(s.usrgrpid IS NULL OR '.dbConditionInt('s.usrgrpid', $options['usrgrpids']).')';
 		}
 
 		// scriptids
 		if ($options['scriptids'] !== null) {
-			zbx_value2array($options['scriptids']);
-
-			$sqlParts['where'][] = dbConditionInt('s.scriptid', $options['scriptids']);
+			$sql_parts['where'][] = dbConditionInt('s.scriptid', $options['scriptids']);
 		}
 
 		// search
-		if (is_array($options['search'])) {
-			zbx_db_search('scripts s', $options, $sqlParts);
+		if ($options['search'] !== null) {
+			zbx_db_search('scripts s', $options, $sql_parts);
 		}
 
 		// filter
-		if (is_array($options['filter'])) {
-			$this->dbFilter('scripts s', $options, $sqlParts);
+		if ($options['filter'] !== null) {
+			$this->dbFilter('scripts s', $options, $sql_parts);
 		}
 
-		// limit
-		if (zbx_ctype_digit($options['limit']) && $options['limit']) {
-			$sqlParts['limit'] = $options['limit'];
-		}
+		$db_scripts = [];
 
-		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
-		while ($script = DBfetch($res)) {
+		$sql_parts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sql_parts);
+		$sql_parts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sql_parts);
+
+		$result = DBselect($this->createSelectQueryFromParts($sql_parts), $options['limit']);
+
+		while ($db_script = DBfetch($result)) {
 			if ($options['countOutput']) {
-				$result = $script['rowscount'];
+				return $db_script['rowscount'];
 			}
-			else {
-				$result[$script['scriptid']] = $script;
+
+			$db_scripts[$db_script['scriptid']] = $db_script;
+		}
+
+		if ($db_scripts) {
+			$db_scripts = $this->addRelatedObjects($options, $db_scripts);
+			$db_scripts = $this->unsetExtraFields($db_scripts, ['scriptid', 'groupid', 'host_access'],
+				$options['output']
+			);
+
+			if (!$options['preservekeys']) {
+				$db_scripts = array_values($db_scripts);
 			}
 		}
 
-		if ($options['countOutput']) {
-			return $result;
-		}
-
-		if ($result) {
-			$result = $this->addRelatedObjects($options, $result);
-			$result = $this->unsetExtraFields($result, ['groupid', 'host_access'], $options['output']);
-		}
-
-		// removing keys (hash -> array)
-		if (!$options['preservekeys']) {
-			$result = zbx_cleanHashes($result);
-		}
-
-		return $result;
+		return $db_scripts;
 	}
 
 	/**
@@ -785,9 +790,7 @@ class CScript extends CApiService {
 		}
 
 		$select_groups = ['name', 'groupid'];
-		if ($options['selectGroups'] !== API_OUTPUT_COUNT) {
-			$select_groups = $this->outputExtend($options['selectGroups'], $select_groups);
-		}
+		$select_groups = $this->outputExtend($options['selectGroups'], $select_groups);
 
 		$host_groups = API::HostGroup()->get([
 			'output' => $select_groups,
