@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -72,6 +72,8 @@ class CTemplate extends CHostGeneral {
 			'editable'					=> false,
 			'nopermissions'				=> null,
 			// filter
+			'evaltype'					=> TAG_EVAL_TYPE_AND_OR,
+			'tags'						=> null,
 			'filter'					=> null,
 			'search'					=> '',
 			'searchByAny'				=> null,
@@ -92,6 +94,7 @@ class CTemplate extends CHostGeneral {
 			'selectMacros'				=> null,
 			'selectScreens'				=> null,
 			'selectHttpTests'			=> null,
+			'selectTags'				=> null,
 			'countOutput'				=> false,
 			'groupCount'				=> false,
 			'preservekeys'				=> false,
@@ -236,6 +239,13 @@ class CTemplate extends CHostGeneral {
 			$sqlParts['where'][] = 'EXISTS (SELECT ht.httptestid FROM httptest ht WHERE ht.hostid=h.hostid)';
 		}
 
+		// tags
+		if ($options['tags'] !== null && $options['tags']) {
+			$sqlParts['where'][] = CApiTagHelper::addWhereCondition($options['tags'], $options['evaltype'], 'h',
+				'host_tag', 'hostid'
+			);
+		}
+
 		// filter
 		if (is_array($options['filter'])) {
 			$this->dbFilter('hosts h', $options, $sqlParts);
@@ -306,6 +316,7 @@ class CTemplate extends CHostGeneral {
 			$templates[$key]['groups'] = zbx_toArray($template['groups']);
 		}
 
+		$ins_tags = [];
 		foreach ($templates as $template) {
 			// if visible name is not given or empty it should be set to host name
 			if ((!isset($template['name']) || zbx_empty(trim($template['name']))) && isset($template['host'])) {
@@ -323,6 +334,12 @@ class CTemplate extends CHostGeneral {
 
 			$templateIds[] = $templateId;
 
+			if (array_key_exists('tags', $template)) {
+				foreach ($template['tags'] as $tag) {
+					$ins_tags[] = ['hostid' => $templateId] + $tag;
+				}
+			}
+
 			foreach ($template['groups'] as $group) {
 				$hostGroupId = get_dbid('hosts_groups', 'hostgroupid');
 
@@ -350,6 +367,10 @@ class CTemplate extends CHostGeneral {
 			}
 		}
 
+		if ($ins_tags) {
+			DB::insert('host_tag', $ins_tags);
+		}
+
 		return ['templateids' => $templateIds];
 	}
 
@@ -357,6 +378,8 @@ class CTemplate extends CHostGeneral {
 	 * Validate create template.
 	 *
 	 * @param array $templates
+	 *
+	 * @throws APIException if the input is invalid.
 	 */
 	protected function validateCreate(array $templates) {
 		$groupIds = [];
@@ -391,6 +414,8 @@ class CTemplate extends CHostGeneral {
 
 		$templateDbFields = ['host' => null];
 
+		$host_name_parser = new CHostNameParser();
+
 		foreach ($templates as $template) {
 			// if visible name is not given or empty it should be set to host name
 			if ((!isset($template['name']) || zbx_empty(trim($template['name']))) && isset($template['host'])) {
@@ -401,11 +426,15 @@ class CTemplate extends CHostGeneral {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('Field "host" is mandatory.'));
 			}
 
-			if (!preg_match('/^'.ZBX_PREG_HOST_FORMAT.'$/', $template['host'])) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s(
-					'Incorrect characters used for template name "%1$s".',
-					$template['host']
-				));
+			// Property 'auto_compress' is not supported for templates.
+			if (array_key_exists('auto_compress', $template)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect input parameters.'));
+			}
+
+			if ($host_name_parser->parse($template['host']) != CParser::PARSE_SUCCESS) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Incorrect characters used for template name "%1$s".', $template['host'])
+				);
 			}
 
 			if (isset($template['host'])) {
@@ -457,6 +486,11 @@ class CTemplate extends CHostGeneral {
 					));
 				}
 			}
+
+			// Validate tags.
+			if (array_key_exists('tags', $template)) {
+				$this->validateTags($template);
+			}
 		}
 	}
 
@@ -494,7 +528,8 @@ class CTemplate extends CHostGeneral {
 
 			$templateCopy = $template;
 
-			$template['templates_link'] = isset($template['templates']) ? $template['templates'] : null;
+			$template['templates_link'] = array_key_exists('templates', $template) ? $template['templates'] : null;
+
 			unset($template['templates'], $template['templateid'], $templateCopy['templates']);
 			$template['templates'] = [$templateCopy];
 
@@ -503,6 +538,8 @@ class CTemplate extends CHostGeneral {
 			}
 		}
 
+		$this->updateTags($templates, 'templateid');
+
 		return ['templateids' => zbx_objectValues($templates, 'templateid')];
 	}
 
@@ -510,6 +547,8 @@ class CTemplate extends CHostGeneral {
 	 * Validate update template.
 	 *
 	 * @param array $templates
+	 *
+	 * @throws APIException if the input is invalid.
 	 */
 	protected function validateUpdate(array $templates) {
 		$dbTemplates = $this->get([
@@ -523,6 +562,16 @@ class CTemplate extends CHostGeneral {
 			if (!isset($dbTemplates[$template['templateid']])) {
 				self::exception(ZBX_API_ERROR_PERMISSIONS, _('You do not have permission to perform this operation.'));
 			}
+
+			// Property 'auto_compress' is not supported for templates.
+			if (array_key_exists('auto_compress', $template)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect input parameters.'));
+			}
+
+			// Validate tags.
+			if (array_key_exists('tags', $template)) {
+				$this->validateTags($template);
+			}
 		}
 	}
 
@@ -555,25 +604,25 @@ class CTemplate extends CHostGeneral {
 		API::Template()->unlink($templateids, null, true);
 
 		// delete the discovery rules first
-		$delRules = API::DiscoveryRule()->get([
-			'output' => ['itemid'],
+		$del_rules = API::DiscoveryRule()->get([
+			'output' => [],
 			'hostids' => $templateids,
 			'nopermissions' => true,
 			'preservekeys' => true
 		]);
-		if ($delRules) {
-			API::DiscoveryRule()->delete(array_keys($delRules), true);
+		if ($del_rules) {
+			API::DiscoveryRule()->delete(array_keys($del_rules), true);
 		}
 
 		// delete the items
-		$delItems = API::Item()->get([
+		$del_items = API::Item()->get([
+			'output' => [],
 			'templateids' => $templateids,
-			'output' => ['itemid'],
 			'nopermissions' => true,
 			'preservekeys' => true
 		]);
-		if ($delItems) {
-			API::Item()->delete(array_keys($delItems), true);
+		if ($del_items) {
+			CItemManager::delete(array_keys($del_items));
 		}
 
 		// delete host from maps
@@ -667,6 +716,65 @@ class CTemplate extends CHostGeneral {
 			API::Application()->delete(array_keys($delApplications), true);
 		}
 
+		// Get host prototype operations from LLD overrides where this template is linked.
+		$lld_override_operationids = [];
+
+		$db_lld_override_operationids = DBselect(
+			'SELECT loo.lld_override_operationid'.
+			' FROM lld_override_operation loo'.
+			' WHERE EXISTS('.
+				'SELECT NULL'.
+				' FROM lld_override_optemplate lot'.
+				' WHERE lot.lld_override_operationid=loo.lld_override_operationid'.
+				' AND '.dbConditionId('lot.templateid', $templateids).
+			')'
+		);
+		while ($db_lld_override_operationid = DBfetch($db_lld_override_operationids)) {
+			$lld_override_operationids[] = $db_lld_override_operationid['lld_override_operationid'];
+		}
+
+		if ($lld_override_operationids) {
+			DB::delete('lld_override_optemplate', ['templateid' => $templateids]);
+
+			// Make sure there no other operations left to safely delete the operation.
+			$delete_lld_override_operationids = [];
+
+			$db_delete_lld_override_operationids = DBselect(
+				'SELECT loo.lld_override_operationid'.
+				' FROM lld_override_operation loo'.
+				' WHERE NOT EXISTS ('.
+						'SELECT NULL'.
+						' FROM lld_override_opstatus los'.
+						' WHERE los.lld_override_operationid=loo.lld_override_operationid'.
+					')'.
+					' AND NOT EXISTS ('.
+						'SELECT NULL'.
+						' FROM lld_override_opdiscover lod'.
+						' WHERE lod.lld_override_operationid=loo.lld_override_operationid'.
+					')'.
+					' AND NOT EXISTS ('.
+						'SELECT NULL'.
+						' FROM lld_override_opinventory loi'.
+						' WHERE loi.lld_override_operationid=loo.lld_override_operationid'.
+					')'.
+					' AND NOT EXISTS ('.
+						'SELECT NULL'.
+						' FROM lld_override_optemplate lot'.
+						' WHERE lot.lld_override_operationid=loo.lld_override_operationid'.
+					')'.
+					' AND '.dbConditionId('loo.lld_override_operationid', $lld_override_operationids)
+			);
+
+			while ($db_delete_lld_override_operationid = DBfetch($db_delete_lld_override_operationids)) {
+				$delete_lld_override_operationids[] = $db_delete_lld_override_operationid['lld_override_operationid'];
+			}
+
+			if ($delete_lld_override_operationids) {
+				DB::delete('lld_override_operation', ['lld_override_operationid' => $delete_lld_override_operationids]);
+			}
+		}
+
+		// Finally delete the template.
 		DB::delete('hosts', ['hostid' => $templateids]);
 
 		// TODO: remove info from API
@@ -744,8 +852,6 @@ class CTemplate extends CHostGeneral {
 				'message' => _('Cannot update templates on discovered host "%1$s".')
 			]));
 
-			$this->validateDependentItemsLinkage($hostIds, $templateIds);
-
 			$this->link($templateIds, $hostIds);
 		}
 
@@ -903,7 +1009,7 @@ class CTemplate extends CHostGeneral {
 			DB::delete('hostmacro', ['hostid' => $templateIds]);
 
 			$this->massAdd([
-				'hosts' => $templates,
+				'templates' => $templates,
 				'macros' => $data['macros']
 			]);
 		}
@@ -1049,11 +1155,12 @@ class CTemplate extends CHostGeneral {
 			}
 		}
 
-		if (isset($data['host']) && !preg_match('/^'.ZBX_PREG_HOST_FORMAT.'$/', $data['host'])) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _s(
-				'Incorrect characters used for template name "%1$s".',
-				$data['host']
-			));
+		$host_name_parser = new CHostNameParser();
+
+		if (array_key_exists('host', $data) && $host_name_parser->parse($data['host']) != CParser::PARSE_SUCCESS) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Incorrect characters used for template name "%1$s".', $data['host'])
+			);
 		}
 	}
 
@@ -1139,10 +1246,9 @@ class CTemplate extends CHostGeneral {
 				]);
 				$templates = zbx_toHash($templates, 'templateid');
 				foreach ($result as $templateid => $template) {
-					if (isset($templates[$templateid]))
-						$result[$templateid]['templates'] = $templates[$templateid]['rowscount'];
-					else
-						$result[$templateid]['templates'] = 0;
+					$result[$templateid]['templates'] = array_key_exists($templateid, $templates)
+						? $templates[$templateid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1169,10 +1275,9 @@ class CTemplate extends CHostGeneral {
 				]);
 				$hosts = zbx_toHash($hosts, 'templateid');
 				foreach ($result as $templateid => $template) {
-					if (isset($hosts[$templateid]))
-						$result[$templateid]['hosts'] = $hosts[$templateid]['rowscount'];
-					else
-						$result[$templateid]['hosts'] = 0;
+					$result[$templateid]['hosts'] = array_key_exists($templateid, $hosts)
+						? $hosts[$templateid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1207,10 +1312,9 @@ class CTemplate extends CHostGeneral {
 				]);
 				$screens = zbx_toHash($screens, 'templateid');
 				foreach ($result as $templateid => $template) {
-					if (isset($screens[$templateid]))
-						$result[$templateid]['screens'] = $screens[$templateid]['rowscount'];
-					else
-						$result[$templateid]['screens'] = 0;
+					$result[$templateid]['screens'] = array_key_exists($templateid, $screens)
+						? $screens[$templateid]['rowscount']
+						: '0';
 				}
 			}
 		}
