@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -211,6 +211,31 @@ class CMacrosResolverHelper {
 		]);
 	}
 
+	/**
+	 * Resolve macros in trigger operational data.
+	 *
+	 * @static
+	 *
+	 * @param array  $trigger
+	 * @param string $trigger['expression']
+	 * @param string $trigger['opdata']
+	 * @param int    $trigger['clock']       (optional)
+	 * @param int    $trigger['ns']          (optional)
+	 * @param array  $options
+	 * @param bool   $options['events']      (optional) Resolve {ITEM.VALUE} macro using 'clock' and 'ns' fields.
+	 *                                       Default: false.
+	 * @param bool   $options['html']        (optional) Default: false.
+	 *
+	 * @return string
+	 */
+	public static function resolveTriggerOpdata(array $trigger, array $options = []) {
+		$triggers = self::resolveTriggerDescriptions([$trigger['triggerid'] => $trigger],
+			$options + ['sources' => ['opdata']]
+		);
+
+		return $triggers[$trigger['triggerid']]['opdata'];
+	}
+
 	/**
 	 * Resolve macros in trigger description.
 	 *
@@ -224,28 +249,33 @@ class CMacrosResolverHelper {
 	 * @param array  $options
 	 * @param bool   $options['events']      (optional) Resolve {ITEM.VALUE} macro using 'clock' and 'ns' fields.
 	 *                                       Default: false.
+	 * @param bool   $options['html']        (optional) Default: false.
 	 *
 	 * @return string
 	 */
 	public static function resolveTriggerDescription(array $trigger, array $options = []) {
-		$triggers = self::resolveTriggerDescriptions([$trigger['triggerid'] => $trigger], $options);
+		$triggers = self::resolveTriggerDescriptions([$trigger['triggerid'] => $trigger],
+			$options + ['sources' => ['comments']]
+		);
 
 		return $triggers[$trigger['triggerid']]['comments'];
 	}
 
 	/**
-	 * Resolve macros in trigger descriptions.
+	 * Resolve macros in trigger descriptions and operational data.
 	 *
 	 * @static
 	 *
 	 * @param array  $triggers
 	 * @param string $triggers[$triggerid]['expression']
-	 * @param string $triggers[$triggerid]['comments']
+	 * @param string $triggers[$triggerid][<sources>]     See $options['sources'].
 	 * @param int    $triggers[$triggerid]['clock']       (optional)
 	 * @param int    $triggers[$triggerid]['ns']          (optional)
 	 * @param array  $options
 	 * @param bool   $options['events']                   (optional) Resolve {ITEM.VALUE} macro using 'clock' and 'ns'
 	 *                                                    fields. Default: false.
+	 * @param bool   $options['html']                     (optional) Default: false.
+	 * @param array  $options['sources']                  An array of trigger field names: 'comments', 'opdata'.
 	 *
 	 * @return array
 	 */
@@ -253,7 +283,8 @@ class CMacrosResolverHelper {
 		self::init();
 
 		$options += [
-			'events' => false
+			'events' => false,
+			'html' => false
 		];
 
 		return self::$macrosResolver->resolveTriggerDescriptions($triggers, $options);
@@ -264,16 +295,19 @@ class CMacrosResolverHelper {
 	 *
 	 * @static
 	 *
-	 * @param array $triggers
-	 * @param string $triggers[triggerid]['expression']
-	 * @param string $triggers[triggerid]['url']
+	 * @param array  $trigger
+	 * @param string $trigger['triggerid']
+	 * @param string $trigger['expression']
+	 * @param string $trigger['url']
+	 * @param string $trigger['eventid']
+	 * @param string $url
 	 *
-	 * @return array
+	 * @return bool
 	 */
-	public static function resolveTriggerUrls(array $triggers) {
+	public static function resolveTriggerUrl(array $trigger, &$url) {
 		self::init();
 
-		return self::$macrosResolver->resolveTriggerUrls($triggers);
+		return self::$macrosResolver->resolveTriggerUrl($trigger, $url);
 	}
 
 	/**
@@ -282,19 +316,17 @@ class CMacrosResolverHelper {
 	 * @static
 	 *
 	 * @param string $expression
-	 * @param array  $options		see resolveTriggerExpressions() for more details ('sources' is not supported here)
+	 * @param array  $options     See CMacrosResolver::resolveTriggerExpressions() for more details.
+	 *                            'sources' is not supported here.
 	 *
 	 * @return string
 	 */
 	public static function resolveTriggerExpression($expression, array $options = []) {
 		self::init();
 
-		return self::$macrosResolver->resolveTriggerExpressions([['expression' => $expression]], [
-			'html' => array_key_exists('html', $options) && $options['html'],
-			'resolve_usermacros' => array_key_exists('resolve_usermacros', $options) && $options['resolve_usermacros'],
-			'resolve_macros' => array_key_exists('resolve_macros', $options) && $options['resolve_macros'],
-			'sources' => ['expression']
-		])[0]['expression'];
+		return self::$macrosResolver->resolveTriggerExpressions(
+			[['expression' => $expression]], $options
+		)[0]['expression'];
 	}
 
 	/**
@@ -302,45 +334,15 @@ class CMacrosResolverHelper {
 	 *
 	 * @static
 	 *
-	 * @param array  $triggers
-	 * @param string $triggers[]['expression']
-	 * @param array  $options
-	 * @param bool   $options['html']				(optional) returns formatted trigger expression
-	 * @param bool   $options['resolve_usermacros']	(optional) resolve user macros
-	 * @param bool   $options['resolve_macros']		(optional) resolve macros in item keys and functions
-	 * @param array  $options['sources']			(optional) an array of the field names; default ['expression']
+	 * @param array $triggers
+	 * @param array $options   See CMacrosResolver::resolveTriggerExpressions() for more details.
 	 *
 	 * @return array
 	 */
 	public static function resolveTriggerExpressions(array $triggers, array $options = []) {
 		self::init();
 
-		return self::$macrosResolver->resolveTriggerExpressions($triggers, [
-			'html' => array_key_exists('html', $options) && $options['html'],
-			'resolve_usermacros' => array_key_exists('resolve_usermacros', $options) && $options['resolve_usermacros'],
-			'resolve_macros' => array_key_exists('resolve_macros', $options) && $options['resolve_macros'],
-			'sources' => array_key_exists('sources', $options) ? $options['sources'] : ['expression']
-		]);
-	}
-
-	/**
-	 * Resolve user macros in trigger expression.
-	 *
-	 * @static
-	 *
-	 * @param array $trigger
-	 * @param array $trigger['triggerid']
-	 * @param array $trigger['expression']
-	 *
-	 * @return string
-	 */
-	public static function resolveTriggerExpressionUserMacro(array $trigger) {
-		self::init();
-
-		$triggers = self::$macrosResolver->resolveTriggerExpressionUserMacro(zbx_toHash([$trigger], 'triggerid'));
-		$trigger = reset($triggers);
-
-		return $trigger['expression'];
+		return self::$macrosResolver->resolveTriggerExpressions($triggers, $options);
 	}
 
 	/**
@@ -416,7 +418,7 @@ class CMacrosResolverHelper {
 
 		$resolvedGraph = reset($graphMap);
 		foreach ($data as &$graph) {
-			if ($graph['graphid'] === $resolvedGraph['graphid']) {
+			if ($resolvedGraph && $graph['graphid'] === $resolvedGraph['graphid']) {
 				$graph['name'] = $resolvedGraph['name'];
 				$resolvedGraph = next($graphMap);
 			}
@@ -491,31 +493,12 @@ class CMacrosResolverHelper {
 	 *
 	 * @return string
 	 */
-	public static function resolveMapLabelMacros($label, $replaceHosts = null) {
+	public static function resolveMapLabelMacros($label, array $replaceHosts = []) {
 		self::init();
 
 		return self::$macrosResolver->resolveMapLabelMacros($label, $replaceHosts);
 	}
 
-	/**
-	 * Resolve all kinds of macros in map labels.
-	 *
-	 * @static
-	 *
-	 * @param array  $selement
-	 * @param string $selement['label']						label to expand
-	 * @param int    $selement['elementtype']				element type
-	 * @param array    $selement['elements']				elements
-	 * @param string $selement['elementExpressionTrigger']	if type is trigger, then trigger expression
-	 *
-	 * @return string
-	 */
-	public static function resolveMapLabelMacrosAll(array $selement) {
-		self::init();
-
-		return self::$macrosResolver->resolveMapLabelMacrosAll($selement);
-	}
-
 	/**
 	 * Resolve macros in dashboard widget URL.
 	 *
@@ -557,9 +540,36 @@ class CMacrosResolverHelper {
 		return self::$macrosResolver->resolveTimeUnitMacros($data, ['sources' => $field_names]);
 	}
 
+	/**
+	 * Resolve supported macros used in map element label as well as in URL names and values.
+	 *
+	 * @static
+	 *
+	 * @param array        $selements[]
+	 * @param int          $selements[]['elementtype']          Map element type.
+	 * @param int          $selements[]['elementsubtype']       Map element subtype.
+	 * @param string       $selements[]['label']                Map element label.
+	 * @param array        $selements[]['urls']                 Map element urls.
+	 * @param string       $selements[]['urls'][]['name']       Map element url name.
+	 * @param string       $selements[]['urls'][]['url']        Map element url value.
+	 * @param int | array  $selements[]['elementid']            Element id linked to map element.
+	 * @param array        $options
+	 * @param bool         $options['resolve_element_urls']     Resolve macros in map element url name and value.
+	 * @param bool         $options['resolve_element_label']    Resolve macros in map element label.
+	 *
+	 * @return array
+	 */
+	public static function resolveMacrosInMapElements(array $selements, array $options) {
+		self::init();
+
+		return self::$macrosResolver->resolveMacrosInMapElements($selements, $options);
+	}
+
 	/**
 	 * Set every trigger items array elements order by item usage order in trigger expression and recovery expression.
 	 *
+	 * @static
+	 *
 	 * @param array  $triggers                            Array of triggers.
 	 * @param string $triggers[]['expression']            Trigger expression used to define order of trigger items.
 	 * @param string $triggers[]['recovery_expression']   Trigger expression used to define order of trigger items.
@@ -573,4 +583,46 @@ class CMacrosResolverHelper {
 
 		return self::$macrosResolver->sortItemsByExpressionOrder($triggers);
 	}
+
+	/**
+	 * Extract macros from properties used for preprocessing step test and find effective values.
+	 *
+	 * @param array  $data
+	 * @param string $data['steps']                              Preprocessing steps details.
+	 * @param string $data['steps'][]['params']                  Preprocessing step parameters.
+	 * @param string $data['steps'][]['error_handler_params]     Preprocessing steps error handle parameters.
+	 * @param string $data['delay']                              Update interval value.
+	 * @param array  $data['supported_macros']                   Supported macros.
+	 * @param bool   $data['support_lldmacros']                  Either LLD macros need to be extracted.
+	 * @param array  $data['texts_support_macros']               List of texts potentially could contain macros.
+	 * @param array  $data['texts_support_user_macros']          List of texts potentially could contain user macros.
+	 * @param array  $data['texts_support_lld_macros']           List of texts potentially could contain LLD macros.
+	 * @param int    $data['hostids']                            Hostid for which tested item belongs to.
+	 *
+	 * @return array
+	 */
+	public static function extractItemTestMacros(array $data) {
+		self::init();
+
+		return self::$macrosResolver->extractItemTestMacros($data);
+	}
+
+	/**
+	 * Return associative array of urls with resolved {EVENT.TAGS.*} macro in form
+	 * [<eventid> => ['urls' => [['url' => .. 'name' => ..], ..]]].
+	 *
+	 * @param array  $events                                Array of event tags.
+	 * @param string $events[<eventid>]['tags'][]['tag']    Event tag tag field value.
+	 * @param string $events[<eventid>]['tags'][]['value']  Event tag value field value.
+	 * @param array  $urls                                  Array of mediatype urls.
+	 * @param string $urls[]['event_menu_url']              Media type url field value.
+	 * @param string $urls[]['event_menu_name']             Media type url_name field value.
+	 *
+	 * @return array
+	 */
+	public static function resolveMediaTypeUrls(array $events, array $urls) {
+		self::init();
+
+		return self::$macrosResolver->resolveMediaTypeUrls($events, $urls);
+	}
 }
