@@ -111,6 +111,11 @@ class CZabbixServer {
 	 */
 	protected $total;
 
+	/**
+	 * @var array $debug  Section 'debug' data from server response.
+	 */
+	protected $debug = [];
+
 	/**
 	 * Class constructor.
 	 *
@@ -170,6 +175,28 @@ class CZabbixServer {
 		]);
 	}
 
+	/**
+	 * Request server to test item.
+	 *
+	 * @param array  $data    Array of item properties to test.
+	 * @param string $sid     User session ID.
+	 *
+	 * @return array
+	 */
+	public function testItem(array $data, $sid) {
+		/*
+		 * Timeout for 'item.test' request is increased because since message can be forwarded from server to proxy and
+		 * later to agent, it might take more time due network latency.
+		 */
+		$this->timeout = 60;
+
+		return $this->request([
+			'request' => 'item.test',
+			'data' => $data,
+			'sid' => $sid
+		]);
+	}
+
 	/**
 	 * Retrieve item queue information.
 	 *
@@ -307,6 +334,38 @@ class CZabbixServer {
 		return CApiInputValidator::validate($api_input_rules, $response, '/', $this->error);
 	}
 
+	/**
+	 * Evaluate trigger expressions.
+	 *
+	 * @param array  $data
+	 * @param string $sid
+	 *
+	 * @return bool|array
+	 */
+	public function expressionsEvaluate(array $data, string $sid) {
+		$response = $this->request([
+			'request' => 'expressions.evaluate',
+			'sid' => $sid,
+			'data' => $data
+		]);
+
+		if ($response === false) {
+			return false;
+		}
+
+		$api_input_rules = ['type' => API_OBJECTS, 'fields' => [
+			'expression' =>	['type' => API_STRING_UTF8, 'flags' => API_REQUIRED],
+			'value' =>		['type' => API_INT32, 'in' => '0,1'],
+			'error' =>		['type' => API_STRING_UTF8]
+		]];
+
+		if (!CApiInputValidator::validate($api_input_rules, $response, '/', $this->error)) {
+			return false;
+		}
+
+		return $response;
+	}
+
 	/**
 	 * Returns the error message.
 	 *
@@ -325,6 +384,15 @@ class CZabbixServer {
 		return $this->total;
 	}
 
+	/**
+	 * Returns debug section from server response.
+	 *
+	 * @return array
+	 */
+	public function getDebug() {
+		return $this->debug;
+	}
+
 	/**
 	 * Executes a given JSON request and returns the result. Returns false if an error has occurred.
 	 *
@@ -336,6 +404,7 @@ class CZabbixServer {
 		// Reset object state.
 		$this->error = null;
 		$this->total = null;
+		$this->debug = [];
 
 		// Connect to the server.
 		if (!$this->connect()) {
@@ -346,7 +415,7 @@ class CZabbixServer {
 		stream_set_timeout($this->socket, $this->timeout);
 
 		// Send the command.
-		$json = CJs::encodeJson($params);
+		$json = json_encode($params);
 		if (fwrite($this->socket, ZBX_TCP_HEADER.pack('V', strlen($json))."\x00\x00\x00\x00".$json) === false) {
 			$this->error = _s('Cannot send command, check connection with Zabbix server "%1$s".', $this->host);
 			return false;
@@ -419,7 +488,7 @@ class CZabbixServer {
 			return false;
 		}
 
-		$response = CJs::decodeJson(substr($response, ZBX_TCP_HEADER_LEN + ZBX_TCP_DATALEN_LEN));
+		$response = json_decode(substr($response, ZBX_TCP_HEADER_LEN + ZBX_TCP_DATALEN_LEN), true);
 
 		if (!$response || !$this->normalizeResponse($response)) {
 			$this->error = _s('Incorrect response received from Zabbix server "%1$s".', $this->host);
@@ -427,6 +496,10 @@ class CZabbixServer {
 			return false;
 		}
 
+		if (array_key_exists('debug', $response)) {
+			$this->debug = $response['debug'];
+		}
+
 		// Request executed successfully.
 		if ($response['response'] == self::RESPONSE_SUCCESS) {
 			// saves total count
@@ -434,12 +507,11 @@ class CZabbixServer {
 
 			return array_key_exists('data', $response) ? $response['data'] : true;
 		}
-		// An error on the server side occurred.
-		else {
-			$this->error = $response['info'];
 
-			return false;
-		}
+		// An error on the server side occurred.
+		$this->error = $response['info'];
+
+		return false;
 	}
 
 	/**
@@ -457,19 +529,19 @@ class CZabbixServer {
 			if (!$socket = @fsockopen($this->host, $this->port, $errorCode, $errorMsg, ZBX_CONNECT_TIMEOUT)) {
 				switch ($errorMsg) {
 					case 'Connection refused':
-						$dErrorMsg = _s("Connection to Zabbix server \"%s\" refused. Possible reasons:\n1. Incorrect server IP/DNS in the \"zabbix.conf.php\";\n2. Security environment (for example, SELinux) is blocking the connection;\n3. Zabbix server daemon not running;\n4. Firewall is blocking TCP connection.\n", $this->host);
+						$dErrorMsg = _s("Connection to Zabbix server \"%1\$s\" refused. Possible reasons:\n1. Incorrect server IP/DNS in the \"zabbix.conf.php\";\n2. Security environment (for example, SELinux) is blocking the connection;\n3. Zabbix server daemon not running;\n4. Firewall is blocking TCP connection.\n", $this->host);
 						break;
 
 					case 'No route to host':
-						$dErrorMsg = _s("Zabbix server \"%s\" can not be reached. Possible reasons:\n1. Incorrect server IP/DNS in the \"zabbix.conf.php\";\n2. Incorrect network configuration.\n", $this->host);
+						$dErrorMsg = _s("Zabbix server \"%1\$s\" can not be reached. Possible reasons:\n1. Incorrect server IP/DNS in the \"zabbix.conf.php\";\n2. Incorrect network configuration.\n", $this->host);
 						break;
 
 					case 'Connection timed out':
-						$dErrorMsg = _s("Connection to Zabbix server \"%s\" timed out. Possible reasons:\n1. Incorrect server IP/DNS in the \"zabbix.conf.php\";\n2. Firewall is blocking TCP connection.\n", $this->host);
+						$dErrorMsg = _s("Connection to Zabbix server \"%1\$s\" timed out. Possible reasons:\n1. Incorrect server IP/DNS in the \"zabbix.conf.php\";\n2. Firewall is blocking TCP connection.\n", $this->host);
 						break;
 
 					default:
-						$dErrorMsg = _s("Connection to Zabbix server \"%s\" failed. Possible reasons:\n1. Incorrect server IP/DNS in the \"zabbix.conf.php\";\n2. Incorrect DNS server configuration.\n", $this->host);
+						$dErrorMsg = _s("Connection to Zabbix server \"%1\$s\" failed. Possible reasons:\n1. Incorrect server IP/DNS in the \"zabbix.conf.php\";\n2. Incorrect DNS server configuration.\n", $this->host);
 				}
 
 				$this->error = $dErrorMsg.$errorMsg;
