@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -32,10 +32,13 @@ jQuery(function($) {
 			TAB: 9
 		};
 
+	const MS_ACTION_POPUP = 0;
+	const MS_ACTION_AUTOSUGGEST = 1;
+
 	/**
 	 * Multi select helper.
 	 *
-	 * @param string options['objectName']		backend data source
+	 * @param string options['object_name']		backend data source
 	 * @param object options['objectOptions']	parameters to be added the request URL (optional)
 	 *
 	 * @see jQuery.multiSelect()
@@ -46,7 +49,7 @@ jQuery(function($) {
 		var curl = new Curl('jsrpc.php', false);
 		curl.setArgument('type', 11); // PAGE_TYPE_TEXT_RETURN_JSON
 		curl.setArgument('method', 'multiselect.get');
-		curl.setArgument('objectName', options.objectName);
+		curl.setArgument('object_name', options.object_name);
 
 		for (var key in options.objectOptions) {
 			curl.setArgument(key, options.objectOptions[key]);
@@ -83,21 +86,36 @@ jQuery(function($) {
 				});
 			}
 
+			// Sort entries by name field.
+			data.sort(function(a, b) {
+				if (a.name === b.name) {
+					return 0;
+				}
+				else {
+					return (a.name < b.name) ? -1 : 1;
+				}
+			});
+
 			return data;
 		},
 
 		/**
 		 * Insert outside data
 		 *
-		 * @param object    multiselect value object
+		 * @param {array} items           Multiselect value object.
+		 * @param {bool}  trigger_change  (optional) Either to trigger element on-change event once data added. True by default.
 		 *
 		 * @return jQuery
 		 */
-		addData: function(items) {
+		addData: function(items, trigger_change) {
 			return this.each(function() {
 				var $obj = $(this),
 					ms = $obj.data('multiSelect');
 
+				if (typeof trigger_change !== 'boolean') {
+					trigger_change = true;
+				}
+
 				if (ms.options.selectedLimit == 1) {
 					for (var id in ms.values.selected) {
 						removeSelected($obj, id);
@@ -108,7 +126,7 @@ jQuery(function($) {
 					addSelected($obj, items[i]);
 				}
 
-				$obj.trigger('change', ms);
+				trigger_change && $obj.trigger('change', ms);
 			});
 		},
 
@@ -263,8 +281,12 @@ jQuery(function($) {
 	 * @param int    options['limit']				how many available items can be received from backend (optional)
 	 * @param object options['popup']				popup data {parameters, width, height} (optional)
 	 * @param string options['popup']['parameters']
+	 * @param string options['popup']['filter_preselect_fields']
 	 * @param int    options['popup']['width']
 	 * @param int    options['popup']['height']
+	 * @param object options['autosuggest']         autosuggest options (optional)
+	 * @param object options['autosuggest']['filter_preselect_fields'] autosuggest preselect fields (optional)
+	 * @param string options['autosuggest']['filter_preselect_fields']['hosts'] autosuggest host preselect field (optional)
 	 * @param string options['styles']				additional style for multiselect wrapper HTML element (optional)
 	 * @param string options['styles']['property']
 	 * @param string options['styles']['value']
@@ -338,12 +360,6 @@ jQuery(function($) {
 				return false;
 			});
 
-			if (IE) {
-				ms.values.available_div.on('mousedown', function() {
-					ms.values.available_false_click = true;
-				});
-			}
-
 			$obj.data('multiSelect', ms);
 
 			$obj.wrap($('<div>', {
@@ -351,7 +367,13 @@ jQuery(function($) {
 				css: ms.options.styles
 			}));
 
-			var $selected_div = $('<div>', {'class': 'selected'}),
+			var $selected_div = $('<div>', {'class': 'selected'}).on('click', function() {
+					/*
+					 * Focus without options because here it don't matter.
+					 * Click used instead focus because in patternselect listen only click.
+					 */
+					$('input[type="text"]', $obj).click().focus();
+				}),
 				$selected_ul = $('<ul>', {'class': 'multiselect-list'});
 
 			$obj.append($selected_div.append($selected_ul));
@@ -369,8 +391,10 @@ jQuery(function($) {
 					if (isSearchFieldVisible($obj) && ms.options.selectedLimit != 1) {
 						$obj.addClass('active');
 						$('.selected li.selected', $obj).removeClass('selected');
-						$('input[type="text"]', $obj).focus();
 					}
+				})
+				.on('remove', function() {
+					cleanSearch($obj);
 				});
 
 			if (empty(ms.options.data)) {
@@ -398,7 +422,15 @@ jQuery(function($) {
 				}
 
 				popup_button.on('click', function(event) {
-					return PopUp('popup.generic', ms.options.popup.parameters, null, event.target);
+					var popup_options = ms.options.popup.parameters;
+
+					if (ms.options.popup.filter_preselect_fields) {
+						popup_options = jQuery.extend(popup_options, getFilterPreselectField($obj, MS_ACTION_POPUP));
+					}
+
+					// Click used instead focus because in patternselect listen only click.
+					$('input[type="text"]', $obj).click();
+					return PopUp('popup.generic', popup_options, null, event.target);
 				});
 
 				$obj.after($('<div>', {'class': 'multiselect-button'}).append(popup_button));
@@ -406,6 +438,40 @@ jQuery(function($) {
 		});
 	};
 
+	/**
+	 * Get current value from preselect filter field.
+	 *
+	 * @param {jQuery} $obj    Multiselect instance.
+	 * @param {int}    action  User action that caused preselection.
+	 *
+	 * @return {object}
+	 */
+	function getFilterPreselectField($obj, action) {
+		var ms = $obj.data('multiSelect'),
+			preselect_options = ms.options[(action == MS_ACTION_AUTOSUGGEST) ? 'autosuggest' : 'popup'] || null,
+			ret = {};
+
+		if (!preselect_options) {
+			return ret;
+		}
+
+		if (typeof preselect_options.filter_preselect_fields.hosts !== 'undefined') {
+			var hosts = $('#' + preselect_options.filter_preselect_fields.hosts).multiSelect('getData');
+			if (hosts.length != 0) {
+				ret.hostid = hosts[0].id;
+			}
+		}
+
+		if (typeof preselect_options.filter_preselect_fields.hostgroups !== 'undefined') {
+			var host_groups = $('#' + preselect_options.filter_preselect_fields.hostgroups).multiSelect('getData');
+			if (host_groups.length != 0) {
+				ret.groupid = host_groups[0].id;
+			}
+		}
+
+		return ret;
+	}
+
 	function makeMultiSelectInput($obj) {
 		var ms = $obj.data('multiSelect'),
 			$label = $('label[for=' + $obj.attr('id') + '_ms]'),
@@ -414,6 +480,7 @@ jQuery(function($) {
 				'id': $label.length ? $label.attr('for') : null,
 				'class': 'input',
 				'type': 'text',
+				'autocomplete': 'off',
 				'placeholder': ms.options.placeholder,
 				'aria-label': ($label.length ? $label.text() + '. ' : '') + ms.options.placeholder,
 				'aria-required': ms.options.required_str
@@ -444,6 +511,9 @@ jQuery(function($) {
 					}
 
 					if (search !== '') {
+						var preselect_values = getFilterPreselectField($obj, MS_ACTION_AUTOSUGGEST),
+							cache_key = search + JSON.stringify(preselect_values);
+
 						/*
 						 * Strategy:
 						 * 1. Load the cached result set if such exists for the given term and show the list.
@@ -451,36 +521,38 @@ jQuery(function($) {
 						 * 3. Schedule result set retrieval for the given term otherwise.
 						 */
 
-						if (search in ms.values.searches) {
+						if (cache_key in ms.values.searches) {
 							ms.values.search = search;
+							ms.values.cache_key = cache_key;
 							loadAvailable($obj);
 							showAvailable($obj);
 						}
-						else if (!(search in ms.values.searching)) {
+						else if (!(cache_key in ms.values.searching)) {
 							ms.values.searchTimeout = setTimeout(function() {
-								ms.values.searching[search] = true;
+								ms.values.searching[cache_key] = true;
 
 								$.ajax({
 									url: ms.options.url + '&curtime=' + new CDate().getTime(),
 									type: 'GET',
 									dataType: 'json',
 									cache: false,
-									data: {
+									data: jQuery.extend({
 										search: search,
-										limit: getLimit($obj)
-									}
+										limit: getLimit($obj),
+									}, preselect_values)
 								})
 									.then(function(response) {
-										ms.values.searches[search] = response.result;
+										ms.values.searches[cache_key] = response.result;
 
 										if (search === $input.val().trim()) {
 											ms.values.search = search;
+											ms.values.cache_key = cache_key;
 											loadAvailable($obj);
 											showAvailable($obj);
 										}
 									})
 									.done(function() {
-										delete ms.values.searching[search];
+										delete ms.values.searching[cache_key];
 									});
 
 								delete ms.values.searchTimeout;
@@ -616,17 +688,17 @@ jQuery(function($) {
 							break;
 					}
 				})
-				.on('focusin', function() {
+				.on('focusin', function($event) {
 					$obj.addClass('active');
 				})
-				.on('focusout', function() {
+				.on('focusout', function($event) {
 					if (ms.values.available_false_click) {
 						ms.values.available_false_click = false;
-						$('input[type="text"]', $obj).focus();
+						$('input[type="text"]', $obj)[0].focus({preventScroll:true});
 					}
 					else {
 						$obj.removeClass('active');
-						$('.selected li:selected', $obj).removeClass('selected');
+						$('.selected li.selected', $obj).removeClass('selected');
 						cleanSearchInput($obj);
 						hideAvailable($obj);
 					}
@@ -658,7 +730,7 @@ jQuery(function($) {
 		addSelected($obj, ms.values.available[id]);
 
 		if (isSearchFieldVisible($obj)) {
-			$('input[type="text"]', $obj).focus();
+			$('input[type="text"]', $obj)[0].focus({preventScroll:true});
 		}
 
 		$obj.trigger('change', ms);
@@ -703,7 +775,7 @@ jQuery(function($) {
 								if (!ms.options.disabled && !item_disabled) {
 									removeSelected($obj, item.id);
 									if (isSearchFieldVisible($obj)) {
-										$('input[type="text"]', $obj).focus();
+										$('input[type="text"]', $obj)[0].focus({preventScroll:true});
 									}
 
 									$obj.trigger('change', ms);
@@ -715,7 +787,6 @@ jQuery(function($) {
 					if (isSearchFieldVisible($obj) && ms.options.selectedLimit != 1) {
 						$('.selected li.selected', $obj).removeClass('selected');
 						$(this).addClass('selected');
-						$('input[type="text"]', $obj).focus();
 					}
 				});
 
@@ -788,14 +859,14 @@ jQuery(function($) {
 
 	function loadAvailable($obj) {
 		var ms = $obj.data('multiSelect'),
-			data = ms.values.searches[ms.values.search];
+			data = ms.values.searches[ms.values.cache_key];
 
 		cleanAvailable($obj);
 
 		var addNew = false;
 
 		if (ms.options.addNew && ms.values.search.length) {
-			if (data.length || objectLength(ms.values.selected) > 0) {
+			if (data.length || objectSize(ms.values.selected) > 0) {
 				var names = {};
 
 				// Check if value exists among available values.
@@ -810,7 +881,7 @@ jQuery(function($) {
 				}
 
 				// Check if value exists among selected values.
-				if (!addNew && objectLength(ms.values.selected) > 0) {
+				if (!addNew && objectSize(ms.values.selected) > 0) {
 					$.each(ms.values.selected, function(i, item) {
 						if (typeof item.isNew === 'undefined') {
 							names[item.name.toUpperCase()] = true;
@@ -833,7 +904,7 @@ jQuery(function($) {
 		var available_more = false;
 
 		$.each(data, function(i, item) {
-			if (ms.options.limit == 0 || objectLength(ms.values.available) < ms.options.limit) {
+			if (ms.options.limit == 0 || objectSize(ms.values.available) < ms.options.limit) {
 				if (typeof ms.values.available[item.id] === 'undefined'
 						&& typeof ms.values.selected[item.id] === 'undefined'
 						&& ms.options.excludeids.indexOf(item.id) === -1) {
@@ -856,13 +927,13 @@ jQuery(function($) {
 		var found = 0,
 			preselected = '';
 
-		if (objectLength(ms.values.available) == 0) {
+		if (objectSize(ms.values.available) == 0) {
 			var div = $('<div>', {
 					'class': 'multiselect-matches',
 					text: ms.options.labels['No matches found']
 				})
 					.on('click', function() {
-						$('input[type="text"]', $obj).focus();
+						$('input[type="text"]', $obj)[0].focus({preventScroll:true});
 					});
 
 			ms.values.available_div.append(div);
@@ -900,7 +971,7 @@ jQuery(function($) {
 					text: ms.options.labels['More matches found...']
 				})
 					.on('click', function() {
-						$('input[type="text"]', $obj).focus();
+						$('input[type="text"]', $obj)[0].focus({preventScroll:true});
 					});
 
 			ms.values.available_div.prepend(div);
@@ -945,7 +1016,7 @@ jQuery(function($) {
 				$(window).height() + $(window).scrollTop() - available_top - obj_padding_y - 10
 			));
 
-		if (objectLength(ms.values.available) > 0) {
+		if (objectSize(ms.values.available) > 0) {
 			available_width_min = Math.max(available_width, 300);
 
 			// Prevent less than 15% width difference for the available list and the input field.
@@ -969,7 +1040,7 @@ jQuery(function($) {
 
 		$available.scrollTop(0);
 
-		if (objectLength(ms.values.available) != 0) {
+		if (objectSize(ms.values.available) != 0) {
 			// Remove selected item selected state.
 			$('.selected li.selected', $obj).removeClass('selected');
 
@@ -1058,6 +1129,7 @@ jQuery(function($) {
 	function cleanSearchHistory($obj) {
 		var ms = $obj.data('multiSelect');
 
+		ms.values.cache_key = '';
 		ms.values.search = '';
 		ms.values.searches = {};
 		ms.values.searching = {};
@@ -1119,19 +1191,7 @@ jQuery(function($) {
 		var ms = $obj.data('multiSelect');
 
 		return (ms.options.limit != 0)
-			? ms.options.limit + objectLength(ms.values.selected) + ms.options.excludeids.length + 1
+			? ms.options.limit + objectSize(ms.values.selected) + ms.options.excludeids.length + 1
 			: null;
 	}
-
-	function objectLength(obj) {
-		var length = 0;
-
-		for (var key in obj) {
-			if (obj.hasOwnProperty(key)) {
-				length++;
-			}
-		}
-
-		return length;
-	}
 });
